#include <PalmOS.h>
#include <SysEvtMgr.h>
#include <chars.h>
#include "GenMemo_res.h"

#include "\Common\constants.h"
#include "\Palm\PalmDb\PalmDb.h"
#include "\Palm\PalmMemoDb\PalmMemoDb.h"
#include "\Palm\PalmHashDb\PalmHashDb.h"
#include "\Palm\PalmPolyDb\PalmPolyDb.h"



/***********************************************************************
 *
 *   Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;
	
typedef struct
{
	UInt8 operationtype;
	UInt16 nummemos;
	UInt16 memosize;
	UInt16 differences;
} MemoGenInfo;

	
typedef StarterAppInfoType* StarterAppInfoPtr;


/***********************************************************************
 *
 *   Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;
UInt32 offset;					//offset within the random database
DmOpenRef RandomDBref;			//random database reference

/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
#define appFileCreator					'strt'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01


// Define the minimum OS version we support
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/


/* FUNCTION: RomVersionCompatible                                           *
 * PURPOSE: This function checks that the ROM version meets the minimum     *
 * requirement for our program. (Supplied by CodeWarrior)                   *
 * DATE: 5/28/02                                                            *
 * CONSTANTS: none                                                          *
 * MODULES: GenMemo                                                         */
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
//			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < sysMakeROMVersion(2,0,0,sysROMStageRelease,0))
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		
		return (sysErrRomIncompatible);
		}

	return (0);
}


/* FUNCTION: GetObjectPtr													*
 * PURPOSE: This routine returns a pointer to an object in the current form.*														
 * DATE: 5/28/02                                                            *
 * CONSTANTS: none                                                          *
 * MODULES: GenMemo                                                         */
 static void *GetObjectPtr(UInt16 objectID)
	{
	FormPtr frmP;
	frmP = FrmGetActiveForm();
	return (FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));
}


/* FUNCTION: MainFormInit                                                                 *
 * PURPOSE: This is a do-nothing function. Simply a placeholder for future expandability. *
 * (Provided by CodeWarrior)                                                              *
 * DATE: 5/28/02                                                                          *
 * CONSTANTS: none                                                                        *
 * MODULES: GenMemo                                                                       */
static void MainFormInit(FormPtr frmP)
{
}


/* FUNCTION: MainFormDoCommand                                                            *
 * PURPOSE: This is a do-nothing function. Simply a placeholder for future expandability. *
 * (Provided by CodeWarrior)                                                              *
 * DATE: 5/28/02                                                                          *
 * CONSTANTS: none                                                                        *
 * MODULES: GenMemo                                                                       */
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	return handled;
}


/* FUNCTION: makeMemo                                                                       *
 * PURPOSE: Given a memo number, a memo size and a handle to the random data record, this   *
 * function will create a memo of the form MEMOXXX YYY...YYY where XXX is the three digit   *
 * memo number and YYY...YYY are random characters whose total length is equal to the       *
 * memo size given. The function will place the memo text in the given character array. The *
 * localoffset parameter is a pointer to a non-global offset variable, to be used when      *
 * this function is called in local mode and global variables are not allowed.              *
 * DATE: 6/11/02                                                                            *
 * CONSTANTS: MEMOSIZE                                                                      *
 * MODULES: GenMemo                                                                         */
void makeMemo( int memonum, int size, MemHandle record, UInt16 *localoffset, char *buffer )
{
	unsigned short y, insert;
	char *randomdata;
	
	if( memonum < 10 )									//number of zeros depends on the memo number (field size is not supported?)
		StrPrintF( buffer, "MEMO#000%d    ", memonum );
	else if( memonum < 100 )
		StrPrintF( buffer, "MEMO#00%d    ", memonum );
	else if( memonum < 1000 )
		StrPrintF( buffer, "MEMO#0%d    ", memonum );
	else if( memonum < 10000 )
		StrPrintF( buffer, "MEMO#%d    ", memonum );

	insert = StrLen( buffer );

	randomdata = MemHandleLock( record );				
	
	if( localoffset != 0 )				//if we are in local mode, we must change the offset that was passed to the function
		for( y = 0; y < size; y++ )						//add the random data from the random database
			buffer[insert++] = randomdata[(*localoffset)++];	//the global variable "offset" scans through the random database
	else								//if we are in global mode, then we can simply change the global offset variable
		for( y = 0; y < size; y++ )						//add the random data from the random database
			buffer[insert++] = randomdata[offset++];	//the global variable "offset" scans through the random database

	MemHandleUnlock( record );
	buffer[insert] = '\0';							//don't forget to add the null terminator the the string
	return;
}


/* FUNCTION: generateMemos
 * PURPOSE: Given the number of memos to generate, the size of each memo and the number of symettric differences from *
 * the memos automatically generated on the PC, this function will create the specified number of memos (in the form  *
 * specified in the makeMemo function header) and put them into the SyncMemo databases (it will also update all the   *
 * meta-data in the SyncMemo databases) as if they were put there manually. NOTE: This overwrites all existing data   *
 * and meta-data in the SyncMemo databases. The last parameter in the arguments list is meant to distiguish between   *
 * global and local operating mode for when this application is called within another one and is unable to use global *
 * variables. If the randomdbref is 0, then global mode applies, other wise the function executes in local mode.      *
 * DATE: 6/21/02                                                                                                      *
 * CONSTANTS: MEMOSIZE                                                                                                *
 *            SyncDocDB, SyncIDocDB, SyncHashDB, SyncPolyDB                                                           *
 * MODULES: GenMemo                                                                                                   */ 
void generateMemos( int number, int size, int differences, UInt8 islocalmode )
{
	DmOpenRef docDb, idocDb, hashDb, polyDb, randomdbref;
	char buffer[MEMOSIZE] = "";								//temporary memo buffer
	unsigned short x;
	MemHandle record;
	LocalID databaseid;
	UInt16 *localoffsetptr, localoffset;
	HashRecordType hashrecorddata;
	
	deleteDatabases();				//first, we will delete all the data and meta-data stored in the SyncMemo application

	openDatabases( &docDb, &idocDb, &hashDb, &polyDb );		//next, we will re-create empty versions of all the databases we deleted
	
	if( !islocalmode )		//if we aren't in local mode, then we can use global vars, and the database is already openend
	{
		record = DmGetRecord( RandomDBref, 0 );					//open the random database record, it will be used to create the memos
		localoffsetptr = 0;	    								//we don't need to use the local offset varaible, b/c we have the global one
	}
	else					//if we are in local mode, then we can't use global vars, and we have to open the database ourselves
	{
		databaseid = DmFindDatabase( 0, "RandomDB" );					//open the Random database
		randomdbref = DmOpenDatabase( 0, databaseid, dmModeReadWrite );
		localoffset = 0;												//the offset points to the current character in the random database
		localoffsetptr = &localoffset;
		record = DmGetRecord( randomdbref, 0 );							//open the record containing the random characters
	}
	
	for( x = 0; x < number; x++ )
	{
			makeMemo( x, size, record, localoffsetptr, buffer );	//the makeMemo function will place the memo text into the buffer
	
			if( (x < number - differences/2 ))				//used to generate symettric differences (don't generate last differences/2 memo numbers)
			{
				hashrecorddata.docindex = saveTestString( buffer, docDb, idocDb );	//save the memo to the memo databases
				hashrecorddata.hashval = calcHashValue(buffer);							//calculate the hash and save it to the hash database
				insertHashIndex( hashrecorddata, hashDb );
				updatePolynomialDb( hashrecorddata.hashval, polyDb, docDb, 0 );		//finally, update the polynomial database to reflect the new memo.
			}
	}
	
	if(!islocalmode)
		DmReleaseRecord( RandomDBref, 0, false );			//we must release the record once we are done with it
	else
	{
		DmReleaseRecord( randomdbref, 0, false );
		DmCloseDatabase( randomdbref );						//we should also close the database, if this app is being called automatically
	}
		
	closeDatabases( docDb, idocDb, hashDb, polyDb);
}
		

/* FUNCTION: generateStandardMemos                                                                                       *
 * PURPOSE: Given the number of memos to generate, the size of each memo and the number of symettric differences from    *
 * the memos on the PC, this function will create the specified number of memos (in the form specified in the makeMemo   *
 * function header) and put them into the PalmPilot's MemoPad application database. NOTE: This function will only create *
 * memos above the number of memos already in the database (ie. if there are 3 memos in the database and 5 memos are     *
 * specified to be created, then only memos #4 #5 and #6 will be appended to the end of the MemoPad database.            *
 * DATE: 6/11/02                                                                                                         *
 * CONSTANTS: MEMOSIZE                                                                                                   *
 *            MemoDB - name of the PalmPilot Memo application database                                                   *
 * MODULES: GenMemo                                                                                                      */
void generateStandardMemos( int number, int size, int differences )
{
	DmOpenRef memoDb;
	char buffer[MEMOSIZE], *dataptr;
	LocalID databaseid;
	int x;
	MemHandle memorecord, record;
	UInt16 temp;
	
	databaseid = DmFindDatabase(0,"MemoDB");				//find the memo database
	if( !databaseid )										//if it is not found, then we can't really do anything
		return;
	memoDb = DmOpenDatabase( 0, databaseid, dmModeReadWrite );	
	
	record = DmGetRecord( RandomDBref, 0 );					//open the random database record, it will be used to create the memos
	for( x = 0; x < number; x++ )
	{
		makeMemo( x, size, record, 0, buffer );				//the "makeMemo" function will create the memo text in the buffer variable
		
		if( ( x < number - differences/2 ))					//used to generate symettric differences (don't generate last differences/2 memo numbers)
		{

			temp = DmNumRecords( memoDb );					//only add memos to the database beyond the number that already exist.
			if( x >= temp )									//(ie. if there are already two memos in the database and we call this function to make
			{												//4 memos, only two memos (the last two created) will be appened to the database)
				temp = dmMaxRecordIndex;					
				memorecord = DmNewRecord( memoDb, &temp, StrLen(buffer)+1 );
				dataptr = MemHandleLock(memorecord);
				DmStrCopy( dataptr, 0, buffer ); 
				MemHandleUnlock(memorecord);
				DmReleaseRecord( memoDb, temp, true );
			}
		}
	}
	DmReleaseRecord( RandomDBref, 0, false );
	DmCloseDatabase( memoDb );
}


/* FUNCTION: addMemo                                                                    *
 * PURPOSE: Given a memo string, this function will add that memo string to the memo    *
 * databases, and it will update all the appropriate meta-data to reflect the addition. *
 * DATE: 6/14/02                                                                        *
 * CONSTANTS: none                                                                      *
 * MODULES: GenMemo                                                                     */
void addMemo( char *memotext )
{
	DmOpenRef docDb, idocDb, hashDb, polyDb;
	HashRecordType hashrecorddata;

	openDatabases( &docDb, &idocDb, &hashDb, &polyDb );

	hashrecorddata.docindex = saveTestString( memotext, docDb, idocDb );	
	hashrecorddata.hashval = calcHashValue(memotext);						
	insertHashIndex( hashrecorddata, hashDb );
	updatePolynomialDb( hashrecorddata.hashval, polyDb, docDb, 0 );
	
	closeDatabases( docDb, idocDb, hashDb, polyDb);
	
	return;
}


/* FUNCTION: MainFormHandleEvent													*
 * PURPOSE: This routine is the event handler for the MainForm of this application.	*
 * (modified from that given by CodeWarrior) If an event is handled by this function*
 * then the function returns true, otherwise it returns false.                      *
 * DATE: 5/28/02                                                                    *
 * CONSTANTS: none                                                                  *
 * MODULES: GenMemo                                                                 */
static Boolean MainFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
    FormPtr frmP;

	switch (eventP->eType) 
	{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;	break;

		case ctlSelectEvent:
			if( eventP ->data.ctlSelect.controlID == Main_SyncMemo_Button )		//EVENT: SyncMemo button pressed
				generateMemos(StrAToI(FldGetTextPtr( GetObjectPtr(Main_Num_Field) ) ), StrAToI( FldGetTextPtr( GetObjectPtr(Main_Size_Field))), StrAToI( FldGetTextPtr( GetObjectPtr(Main_Diff_Field))), 0);
			else																//EVENT: Standard Memo button pressed
				generateStandardMemos( StrAToI(FldGetTextPtr( GetObjectPtr(Main_Num_Field) ) ), StrAToI( FldGetTextPtr( GetObjectPtr(Main_Size_Field))), StrAToI( FldGetTextPtr( GetObjectPtr(Main_Diff_Field))));
			
			handled = true; break;
			
		default:
			break;
	}
	return handled;
}


/* FUNCTION: AppHandleEvent	        												*
 * PURPOSE: This routine is the event handler for the application. (modified from   *
 * that given by CodeWarrior) If an event is handled by this function then the      *
 * function returns true, otherwise it returns false.                               *
 * DATE: 5/28/02                                                                    *
 * CONSTANTS: none                                                                  *
 * MODULES: GenMemo                                                                */
static Boolean AppHandleEvent( EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case Main_Form:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	return false;
}


/* FUNCTION: AppEventLoop												                  *
 * PURPOSE: This routine is the event loop for the application. (Provided by CodeWarrior) *
 * DATE: 5/28/02                                                                          *
 * CONSTANTS: none                                                                        *
 * MODULES: GenMemo                                                                       */
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do 
	{
		EvtGetEvent(&event, evtWaitForever);
			
		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/* FUNCTION: AppStart														  *
 * PURPOSE: Get the current application's preferences, and open its databases *
 * (modified by function provided by CodeWarrior)		                      *
 * DATE: 5/28/02                                                              *
 * CONSTANTS: none                                                            *
 * MODULES: GenMemo                                                           *
 * SUGGESTION: Why is the Random Database opened for Reading AND Writing?     */
static Err AppStart(void)
{
    StarterPreferenceType prefs;
    UInt16 prefsSize;
    LocalID databaseid; 

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}

	databaseid = DmFindDatabase( 0, "RandomDB" );					//open the Random database
	RandomDBref = DmOpenDatabase( 0, databaseid, dmModeReadWrite );
	offset = 0;														//the offset points to the current character in the random database
    return 0;
}


/* FUNCTION: AppStop														  *
 * PURPOSE: Save the current state of the application and close its databases *
 * (modified by function provided by CodeWarrior)		                      *
 * DATE: 5/28/02                                                              *
 * CONSTANTS: none                                                            *
 * MODULES: GenMemo                                                           */
static void AppStop(void)
{
   StarterPreferenceType prefs;
      
	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	DmCloseDatabase( RandomDBref );
}


/* FUNCTION: StarterPilotMain                                                 *
 * PURPOSE: This is the main entry point of the application. It checks to see *
 * if the proper version of the OS is installed and starts up the application.*
 * It also calls the event loop and the application close code.               *
 * DATE: 5/28/02                                                              *
 * CONSTANTS: none                                                            *
 * MODULES: GenMemo                                                           */
static UInt32 StarterPilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error;
	MemoGenInfo *memogeninfo;
	char *memotext;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(Main_Form);
			AppEventLoop();
			AppStop();
			break;

		//this command is for when GenMemo is launched by PalmAutoTest to automatically generate memos
		case 99:
			memogeninfo = cmdPBP;
			generateMemos( memogeninfo->nummemos, memogeninfo->memosize, memogeninfo->differences, 1); 
			break;
			
		//this command is for when GenMemo is launched by PalmAutoTest to add a single specific memo
		case 100:
			memotext = cmdPBP;
			addMemo( memotext );
			break;

		default:
			break;
		}
	return 0;
}


/* FUNCTION: PilotMain																	*
 * PURPOSE: This is the main entry point for the application. (provided by CodeWarrior) *
 * DATE: 5/28/02                                                                        *
 * CONSTANTS: none                                                                      *
 * MODULES: GenMemo                                                                     */
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return StarterPilotMain(cmd, cmdPBP, launchFlags);
}

