#include <PalmOS.h>
#include <DLServer.h>
#include <chars.h>
#include "SyncMemoRsc.h"

#include "\Common\constants.h"
#include "\Palm\PalmDb\PalmDb.h"
#include "\Palm\PalmCond\PalmCond.h"
#include "\Palm\PalmMemoDb\PalmMemoDb.h"
#include "\Palm\PalmHashDb\PalmHashDb.h"
#include "\Palm\PalmPolyDb\PalmPolyDb.h"


/*							*
 * Type Defined Structures 	*
 *							*/
typedef struct 
	{
	UInt8 replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;


typedef StarterAppInfoType* StarterAppInfoPtr;



/*						*
 * Global Variables		*
 *						*/	
DmOpenRef docDBref;				//global database where all memo's are stored in alphabetical order
DmOpenRef idocDBref;			//global database where all memos are stored in insetion order
DmOpenRef hashDBref;			//global database where all hash values for memos are stored
DmOpenRef polyDBref;			//global database where the characteristic polynomials are stored
Int16     document;				//document # (record index) of current memo (-1 for new) in SyncDocDB



/*				*
 * Constants	*
 *				*/
#define appFileCreator             'strt'
#define appVersionNum              0x01
#define appPrefID                  0x00
#define appPrefVersionNum          0x01
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)   // Define the minimum OS version we support



/*						*
 * Function Prototypes	*
 *						*/
void LoadDocument( FieldPtr fieldptr, int docnumber );
void RemoveDocument( int docnumber, FieldType *activefield );
void deleteDocument( int docnumber, FieldType *activefield );



/* FUNCTION: RomVersionCompatible                                           *
 * PURPOSE: This function checks that the ROM version meets the minimum     *
 * requirement for our program. (Supplied by CodeWarrior)                   *
 * DATE: 5/23/02                                                            *
 * CONSTANTS: none                                                          *
 * MODULES: SyncMemo                                                        */
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < sysMakeROMVersion(2,0,0,sysROMStageRelease,0))
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		
		return (sysErrRomIncompatible);
		}
	return (0);
}


/* FUNCTION: GetObjectPtr													*
 * PURPOSE: This routine returns a pointer to an object in the current form.*														
 * DATE: 5/23/02                                                            *
 * CONSTANTS: none                                                          *
 * MODULES: SyncMemo                                                        */
static void *GetObjectPtr(UInt16 objectID)
	{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return (FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));
}


/* FUNCTION: EditFormInit																		*
 * PURPOSE: Function loads the record whose index (in the document database) is equal to the    *
 * global variable "document" at the time of the function call, or just a blank document if     *
 * "document" = -1	                                                                            *
 * DATE: 6/20/02                                                                                *
 * CONSTANTS: none                                                                              *
 * MODULES: SyncMemo                                                                            */
static void EditFormInit(FormPtr frmP)
{
	FieldPtr fieldptr;
	
	if( numNonDeletedMemos(docDBref) != 0 )		//if there are existing documents in the document database, load them into the edit field
	{
		fieldptr = (FieldPtr)FrmGetObjectPtr( FrmGetActiveForm(), FrmGetObjectIndex( FrmGetActiveForm(), Edit_TextField ) );

		if( document != -1 )
			LoadDocument( fieldptr, document );
	}	
}


/* FUNCTION: ListDrawFunc                                               *
 * PURPOSE:	This function is called automatically by the Palm for every *
 * item drawn in the list. It is responsible for retrieving the data to *  
 * be drawn (ie. the first 27 characters of the memo), and displaying it*
 * DATE: 6/20/02                                                        *
 * CONSTANTS: 28 - number of characters to display from title           *
 * MODULES: SyncMemo                                                    */
void ListDrawFunc( UInt16 itemNum, RectanglePtr bounds, char *data )
{
	char *record;
	static char buffer[28] = "";

	record = getMemoText( docDBref, itemNum, NULL );

	if( StrLen(record) > 25 )		//if the memo is longer than 25 characters, then we must append a null terminator to it
	{
		StrNCopy(buffer, record, 25);
		buffer[24] = '.'; buffer[25] = '.'; buffer[26] = '.'; buffer[27] = '\0';
		WinDrawChars(buffer, 27, bounds->topLeft.x, bounds->topLeft.y);
	}
	else
		WinDrawChars(record, StrLen(record), bounds->topLeft.x, bounds->topLeft.y);
}


/* FUNCTION: FileFormInit											                     *
 * PURPOSE: This routine initializes the FileForm form which mostly involves setting the *
 * list up with a callback function to draw the first parts of the memos.                *
 * DATE: 6/20/02                                                                         *
 * CONSTANTS: 10 - the number of lines of memo titles that can be displayed on one screen*
 * MODULES: SyncMemo                                                                     */
static void FileFormInit(FormPtr frmP)
{
	UInt16 numrecs, listid;
	ListType *listp;

	listid = FrmGetObjectIndex(frmP, File_FileList);		//get a ptr to the list structure in the File Form
	listp = (ListType *)FrmGetObjectPtr( frmP, listid );
		
	if( (numrecs = numNonDeletedMemos(docDBref)) != 0 )		//if there are non-deleted records in the database
	{
		LstSetListChoices( listp, NULL, numrecs );				//then set the number of list items
		if( numrecs <= 10 )										//the height of the list (not > 9)
			LstSetHeight(listp, numrecs);
		else
			LstSetHeight(listp, 10);
		LstSetDrawFunction(listp, (ListDrawDataFuncPtr)ListDrawFunc );	//then set the callback function to draw
		LstDrawList(listp);												//each record and then redraw the list
	}
	else						//if there are no records, just load the Edit form with a blank document and skip the File Form
	{
		FrmGotoForm(Edit_Form);
		document = -1;
	}		
}


/* FUNCTION: MainFormDoCommand										                 *
 * PURPOSE: This routine performs the menu command specified. (CodeWarrior supplied) *
 * DATE: 5/23/02                                                                     *
 * CONSTANTS: none                                                                   *
 * MODULES: SyncMemo                                                                 */
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;

	switch (command)
		{
		case MainOptionsAboutStarterApp:
			MenuEraseStatus (0);
			AbtShowAbout (appFileCreator);
			handled = true;
			break;
		}
	return handled;
}


/* FUNCTION: saveDocument                                                               *
 * PURPOSE: This function will save the text that is currently in the edit field of the *
 * Edit form as a memo. If the memo has been loaded from memory, its old copy will be   *
 * deleted and a new copy made. All the meta-data databases will be updated as well.    *
 * DATE: 6/20/02                                                                        *
 * CONSTANTS: MEMOSIZE                                                                  *
 * MODULES: SyncMemo                                                                    */
void saveDocument( FieldPtr fieldptr )
{
	char *data, buffer[MEMOSIZE];
	HashRecordType hashrecorddata;
	UInt16 index, hashvalue;
	
	data = FldGetTextPtr(fieldptr);				//get a pointer to the edit field containing the memo string
	
	if( data != NULL && StrLen(data) != 0 )		//if there is data to be saved (don't save a blank memo)
	{
		StrNCopy( buffer, data, MEMOSIZE );
		if( document != -1 )					//if memo already existed (if it was loaded from the document database), check its hash value
		{
			index = findRecordWithDocIndex( document, hashDBref );	//get the hash value of the stored (non-updated) version of the memo
			hashvalue = getHashValue( hashDBref, index );

			if( hashvalue != calcHashValue( buffer ) )			//if the stored hash value is different from a newly calculated value
				deleteDocument( document, fieldptr );				//then we can assume that the memo has been updated (changed) and we'll delete the old copy
			else												//if the stored hash value is the same as the newly calculated value
				return;												//then we can assume that the memo hasn't been updated (changed), so don't save it
		}
		
		hashrecorddata.docindex = saveTestString( buffer, docDBref, idocDBref );	//save the memo to the memo databases
		hashrecorddata.hashval = calcHashValue(buffer);								//calculate the hash and save it to the hash database
		insertHashIndex( hashrecorddata, hashDBref );
		updatePolynomialDb( hashrecorddata.hashval, polyDBref, docDBref, 0 );		//finally, update the polynomial database to reflect the new memo.
	} 
}


/* FUNCTION: RemoveDocument                                                                       *
 * PURPOSE: This function will erase a memo from the memo database, and remove all its associated *
 * meta-data from the other databases.                                                            *
 * DATE: 6/20/02                                                                                  *
 * CONSTANTS: 1 - stands for deletion when passed to the updatePolynomialDb function              *
 * MODULES: SyncMemo                                                                              */
void RemoveDocument( int docnumber, FieldType *activefield )
{
	UInt16 hash, hashindex;
	
	if( docnumber != -1 )		//if somehow, an unsaved document was specified to be deleted, just ignore it.
	{
		if( activefield != NULL )										//dissassociate the field from the document record
			FldSetText( activefield, NULL, 0, 0 );

		hash = calcHashValue( getMemoText(docDBref, docnumber, NULL ) );		//before removing the document text get its hash value

		removeMemoFromDocDb( docDBref, docnumber );						//and then completely remove the document record from the document database

		docnumber = findIndexWithHashVal( hash, hashDBref, &hashindex );//go through the hash database and bump down every document index
		decrementDocPtrsGreaterThan( hashDBref, docnumber );
		
		deleteHashIndex( hashindex, hashDBref );						//delete the hash entry from the hash array
		
		removeMemoFromiDocDb( idocDBref, docnumber );					//remove the memo text from the insertion order document database
	
		updatePolynomialDb( hash, polyDBref, docDBref, 1); 				//update the polynomial database to reflect the deletion
	}
}


/* FUNCTION: deleteDocument
 * PURPOSE: This function sets a document up to be deleted at the next      *
 * hotsync. We must, therefore, maintain the meta-data so that any          *
 * synchronization can detect that the document is present before deciding  *
 * whether or not to delete it. However, the meta-data, (specifically the   *
 * data in the hash database) needs to be modified and the memo text record *
 * needs to be moved to the end of the document database.                   *
 * DATE: 5/23/02                                                            *
 * CONSTANTS: none                                                          *
 * MODULES: SyncMemo                                                        *
 * SUGGESTION: THIS	FUNCTION DOES NOT WORK YET!! IT MUST DELETE THE MEMO    *
 *             FROM THE INSERTION ORDER DOCUMENT DATABASE TOO!              */
void deleteDocument( int docnumber, FieldType *activefield )
{
	int index;
	
	if( document != -1 )			//if somehow, an unsaved document was specified to be deleted, ignore it.
	{
		if( activefield != NULL )	//dissassociate the field from the document record
			FldSetText( activefield, NULL, 0, 0 );

		deleteMemoFromDocDb( docDBref, docnumber );		//mark the memo as deleted in the document database

		index = decrementDocPtrsGreaterThan( hashDBref, docnumber );	//update the document pointers in the hash database (bump them down one)

		changeDocPtrAtHashIndex( hashDBref, index, numTotalMemos( docDBref ) );	//update the document ptr of hash index corresponding to the deleted memo
	}
}	


/* FUNCTION: loadDocument
 * PURPOSE: Given a pointer to the edit field in the Edit form and an index, this *
 * function will load the text at the record index in the document database into  *
 * the edit field.                                                                *
 * DATE: 6/20/02                                                                  *
 * CONSTANTS: MEMOSIZE                                                            *
 * MODULES: SyncMemo                                                              */
void LoadDocument( FieldPtr fieldptr, int docnumber )
{
	fieldptr = (FieldPtr)FrmGetObjectPtr( FrmGetActiveForm(), FrmGetObjectIndex( FrmGetActiveForm(), Edit_TextField ) );
		
	FldSetText(fieldptr, getMemoHandle(docDBref, docnumber), 0, getMemoTextLength(docDBref, docnumber)+1);	//this function places a lock on the chunk so it will not be moved until "saved"
	FldDrawField(fieldptr);
}


/* FUNCTION: EditFormHandleEvent													*
 * PURPOSE: This routine is the event handler for the EditForm of this application.	*
 * (modified from that given by CodeWarrior) If an event is handled by this function*
 * then the function returns true, otherwise it returns false.                      *
 * DATE: 5/23/02                                                                    *
 * CONSTANTS: none                                                                  *
 * MODULES: SyncMemo                                                                */
static Boolean EditFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;		//start with the event NOT having been handled
	FormPtr frmP;
	FieldPtr fieldptr;
	char *data = NULL;
	UInt16 textid, checkboxindex, button;
	Boolean deleteme = true;
	FormType *form;


	switch (eventP->eType) 
	{
		case menuEvent:												//EVENT: a menu item was clicked
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:											//EVENT: Edit form was just opened
			frmP = FrmGetActiveForm();
			FrmDrawForm ( frmP);
			EditFormInit( frmP);

			textid = FrmGetObjectIndex(frmP, Edit_TextField); 		//get text field ptr
			fieldptr = (FieldPtr)FrmGetObjectPtr (frmP, textid); 

			if( document != -1 )									//load the document, if there is one
				LoadDocument( fieldptr, document );
			
			FrmSetFocus(frmP,textid); 								//set focus to the edit field so user can just start entering text
			handled = true;	break;

		case fldChangedEvent:										//EVENT: main text field was scrolled
			handled = true;	break;
			 
		case appStopEvent:											//EVENT: application is switched off
			fieldptr = (FieldPtr)FrmGetObjectPtr( FrmGetActiveForm(), FrmGetObjectIndex( FrmGetActiveForm(), Edit_TextField ) );
			saveDocument(fieldptr);		                            //save the document currently being edited
			handled = true; break;
			
		case ctlSelectEvent:										//EVENT: a control (button) was pressed
			if( eventP ->data.ctlSelect.controlID == Edit_DoneButton )	//the "DONE" button was pressed
			{
				fieldptr = (FieldPtr)FrmGetObjectPtr( FrmGetActiveForm(), FrmGetObjectIndex( FrmGetActiveForm(), Edit_TextField ) );
				saveDocument(fieldptr);
				FrmGotoForm(File_Form);
			}
			else if( eventP ->data.ctlSelect.controlID == Edit_DeleteButton )	//the "DELETE" button was pressed
			{
				form = FrmInitForm( Delete_Form );					//get the delete dialog box ready and display it
				
				checkboxindex = FrmGetObjectIndex( form, Delete_DeletemeCheckbox);
				FrmSetControlValue( form, checkboxindex, deleteme );
				button = FrmDoDialog( form );
				
				deleteme = FrmGetControlValue( form, checkboxindex );	//get the users input to the delete dialog box
				
				FrmDeleteForm(form);
				
				if( button != Delete_CancelButton )		            //execute the appropriate action
				{
					if( deleteme )
						deleteDocument( document, (FieldPtr)GetObjectPtr( Edit_TextField ) );
					else
						RemoveDocument( document, (FieldPtr)GetObjectPtr( Edit_TextField ) );
					
					FrmGotoForm(File_Form);                         //then go back to the File selection form
				}
			}
			handled = true;	break;
			
		case frmCloseEvent:											//EVENT: Form is closing
			fieldptr = (FieldPtr)FrmGetObjectPtr( FrmGetActiveForm(), FrmGetObjectIndex( FrmGetActiveForm(), Edit_TextField ) );
			FldSetText( fieldptr, NULL, 0, 0 );			//dissassociate the text in the field with the field itself 

			break;										//we need not set handled to true because we want to exit the prog.
			
		default: break;
	}
	return handled;
}


/* FUNCTION: FileFormHandleEvent													*
 * PURPOSE: This routine is the event handler for the File form of this application.*
 * (modified from that given by CodeWarrior) If an event is handled by this function*
 * then the function returns true, otherwise it returns false.                      *
 * DATE: 7/3/02                                                                     *
 * CONSTANTS: none                                                                  *
 * MODULES: SyncMemo                                                                */
static Boolean FileFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (eventP->eType) 
	{
		case frmOpenEvent:											//EVENT: File form is opened
			frmP = FrmGetActiveForm();
			FrmDrawForm ( frmP);
			FileFormInit( frmP);
			handled = true;	break;

		case lstSelectEvent:										//EVENT: File selected from the list
			document = eventP->data.lstSelect.selection;			//set the global "document" variable to the proper index
			FrmGotoForm(Edit_Form);
			handled = true;	break;
			
		case ctlSelectEvent:										//EVENT: A control (button) was pressed
			if( eventP ->data.ctlSelect.controlID == File_NewButton )	//the "New Memo" button was pressed
			{
				document = -1;											//a new file is denoted with global "document" variable = -1
				FrmGotoForm(Edit_Form);				
			}
			else if( eventP ->data.ctlSelect.controlID == File_DiagnosticDumpButton )		//add code here for potential future diagnostic
			{
			}
			handled = true;	break;

		default: break;	
	}
	return handled;
}


/* FUNCTION: AppHandleEvent																      *
 * PURPOSE: This routine loads form resources and sets the event handler for the loaded form. *
 * (modified from function provided by CodeWarrior)                                           *
 * DATE: 5/23/02                                                                              *																																																																																																				
 * CONSTANTS: none                                                                            *
 * MODULES: SyncMemo                                                                          */
static Boolean AppHandleEvent( EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)					//EVENT: load a form for the app.
	{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)			//the two main windows (Edit and File) each have thier own event handlers
		{
			case Edit_Form:
				FrmSetEventHandler(frmP, EditFormHandleEvent); break;

			case File_Form:
				FrmSetEventHandler(frmP, FileFormHandleEvent); break;
				
			default: break;
		}
		return true;			//if the event was a form load event, then it was handled
	}
	return false;				//if it wasn't, then just pass it on
}


/* FUNCTION: AppEventLoop												                  *
 * PURPOSE: This routine is the event loop for the application. (Provided by CodeWarrior) *
 * DATE: 5/23/02                                                                          *
 * CONSTANTS: none                                                                        *
 * MODULES: SyncMemo                                                                      */
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do
	{
		EvtGetEvent(&event, evtWaitForever);
		
		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/* FUNCTION: AppStart														  *
 * PURPOSE: Get the current application's preferences, and open its databases *
 * (modified by function provided by CodeWarrior)		                      *
 * DATE: 5/23/02                                                              *
 * CONSTANTS: none                                                            *
 * MODULES: SyncMemo                                                          */
static Err AppStart(void)
{
    StarterPreferenceType prefs;
    UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}

	openDatabases( &docDBref, &idocDBref, &hashDBref, &polyDBref );		//open the global databases for the app.
    return 0;
}


/* FUNCTION: AppStop														  *
 * PURPOSE: Save the current state of the application and close its databases *
 * (modified by function provided by CodeWarrior)		                      *
 * DATE: 5/23/02                                                              *
 * CONSTANTS: none                                                            *
 * MODULES: SyncMemo                                                          */
static void AppStop(void)
{
   StarterPreferenceType prefs;
    
	// Write the saved preferences / saved-state information.  This data 
	// will be backed up during a HotSync.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
	
	closeDatabases( docDBref, idocDBref, hashDBref, polyDBref );		//close the global databases for the app.
}


/* FUNCTION: StarterPilotMain							                                        *
 * DESCRIPTION: This is the main entry point for the application. It processes both the normal  * 
 * launch code (sysAppLaunchCmdNormalLaunch) by starting the application, and the remote launch * 
 * code (sysAppLaunchCmdHandleSyncCallApp) by calling the palmRPC function.                     *
 * DATE: 6/20/02                                                                                *
 * CONSTANTS: none                                                                              *
 * MODULES: SyncMemo                                                                            */
static UInt32 StarterPilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	Err error;
	UInt16  deletedindex = 0;
	DmOpenRef docDB = NULL, idocDB = NULL, hashDB = NULL, polyDB = NULL;
	
	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
	{
		case sysAppLaunchCmdNormalLaunch:			//NORMAL LAUNCH: load the file selection form, start event loop
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(File_Form);
			AppEventLoop();
			AppStop();
	
			FrmCloseAllForms();		
			break;

		case sysAppLaunchCmdHandleSyncCallApp:		//REMOTE LAUNCH: call the Remote Procedure Call of the Palm
			palmRPC( cmdPBP );		                
			break;
		
		default: break;
	}
	return 0;
}


/* FUNCTION: PilotMain																	*
 * PURPOSE: This is the main entry point for the application. (provided by CodeWarrior) *
 * DATE: 5/23/02                                                                        *
 * CONSTANTS: none                                                                      *
 * MODULES: SyncMemo                                                                    */
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return StarterPilotMain(cmd, cmdPBP, launchFlags);
}
