#include "PCMemoFile.h"


/* FUNCTION: deleteRecord                                                              *
 * PURPOSE: Given a pointer to the array of memos, a memo's hash value, and the number *
 * of memos in the array pointed to, this function will go linearly through the array  *
 * and delete the first memo it finds who's hash value matches the given one. It will  *
 * then bump all the memo's after the deleted one down 1 index so that the array will  *
 * not have "holes" in it.                                                             *
 * DATE: 5/22/02                                                                       *
 * CONSTANTS: none                                                                     *
 * MODULES: PCMemoFile                                                                 */
void deleteRecord( memo *memodata, int hash, int nummemos )
{
	for( int x = 0; x < nummemos; x++ )				//loop through all memos in the array
		if( memodata[x].hash == hash )				//if you find the memo
		{
			for( int y = x; y < nummemos-1; y++ )	//bump down all memos after it (thus overwriting it)
				copyMemoEntry( &(memodata[y+1]), &(memodata[y]) );
			return;
		}
}


/* FUNCTION: getStringWithHash                                           *
 * PURPOSE: Given a hash value to look for and a pointer to an array of  *
 * memos, this function will search linearly through the array and       *
 * return the data string of the first memo whose hash value matches that*
 * given. If no match is found, then a blank string is returned.         *
 * DATE: 6/3/02                                                          *
 * CONSTANTS: none                                                       *
 * MODULES: PCMemoFile                                                   */
char *getStringWithHash(int hashvalue, memo *memofile )
{
	for( int x = 0; memofile[x].hash != 0; x++ )	//loop through all valid memos
		if( memofile[x].hash == hashvalue )
			return(memofile[x].data);				//return once proper memo is found

	return("");										//only errors reach this point!
}


/* FUNCTION: loadFile                                                        *
 * PURPOSE: Given an array of type "memo" and a path to the PC's memo file   *
 * this function will read into the memo array all the memos stored in the   *
 * PC's memo file. THIS WILL OVERWRITE ANY EXISTING MEMOS IN THE MEMO ARRAY! *
 * This function returns the number of memos stored in the file.             *
 * DATE: 5/22/02                                                             *
 * CONSTANTS: none                                                           *
 * MODULES: PCMemoFile                                                       */
int loadFile( memo *memodata, char *pcMemoFilePath )
{
	short size;
	long temppos, strpos;
	short numberstored;
	FILE *input = 0;

	
	freeData( memodata );	
	initializeNewData( memodata );

	input = fopen( pcMemoFilePath, "rb" );		//open the PC's data file
	if( input == NULL )							//if it doesn't exist at the location specified, return 0
		return(0);

	fread( &numberstored, sizeof(short), 1, input );	//the first short tells how many memos stored in this file

	for( int x = 0; x < numberstored; x++ )				//for each memo stored
	{
		fread( &memodata[x].hash, sizeof(short), 1, input);		//get its hashvalue
		fread( &size, sizeof(short), 1, input );				//get its length (in characters)
		fread( &strpos, sizeof(long), 1, input );				//get the position of its data string in this file (byte offset)

		temppos = ftell(input);									//keep track of the current location, so we can get back to it
		fseek( input, strpos, SEEK_SET );						//go to the location of the data string
		memodata[x].data = new char[size+1];					//allocate space for the string
		fread( memodata[x].data, size+1, 1, input );			//read the data string
		fseek( input, temppos, SEEK_SET );						//return to the original location to read the next memos data
	}

	fclose(input);
	return( numberstored );
}


/* FUNCTION: saveFile               
 * PURPOSE: This function will store memos in a file, given the desired *
 * file name/location, a pointer to the memos to store, and the number  *
 * of memos to store.                                                   *
 * DATE: 5/22/02                                                        *
 * CONSTANTS: NUMMEMOS                                                  *
 * MODULES: PCMemoFile                                                  *
 * SUGGESTION: Only call fflush once before closing the file            */
void saveFile( char *pcMemoFilePath, int nummemos, memo *memodata)
{
	long pos = 0;
	long filepos[NUMMEMOS];
	short length = 0;

	FILE *output = fopen(pcMemoFilePath, "wb" );		//overwrite any existing file of the same name

	fwrite( &nummemos, sizeof(short), 1, output );		//first, we must store the number of memos to be in the file

	for( int x = 0; x < nummemos; x++ )					//for each memo, store its hashvalue, length and position in the file
	{
		fwrite(&(memodata[x].hash), sizeof(short), 1, output );
		length = strlen(memodata[x].data);
		fwrite( &length, sizeof(short), 1, output );
		fwrite(&pos, sizeof(long), 1, output );				//store a dummy position for now (just to keep the space
		fflush(output);										//clear) We will get the exact location of the data string next
	}

	for( x = 0; x < nummemos; x++ )						//after the hashvalue, length and posistions of every memo have
	{                                                   //been written, we can store each memo's data string
		filepos[x] = ftell(output);							//we keep track of the offset location in the file of each string
		fwrite(memodata[x].data, strlen(memodata[x].data) + 1, 1, output );
		fflush(output);
	}

	for( x = 0; x < nummemos; x++ )						//for each memo, fill in the position offset from the previous step
	{
		fseek(output, 8*x + 6, SEEK_SET );
		fwrite( &(filepos[x]), sizeof(long), 1, output );
		fflush(output);
	}

	fclose(output);
}
