/* PROGRAM: ConstChange                                                            *
 * PURPOSE: This program will open a text file of a specific format and change the *
 * value of a #define'd constant within it. This program is meant to assist in the *
 * automatic testing of the SyncMemo project by allowing the fundamental constants *
 * in the project to be changed and recompiled (seperately from this program) as   *
 * needed by the tester.                                                           *
 * USAGE: ConstChange <file name> <constant name> <operation> <optional>           *
 * <file name> - the fully qualified path name to the file containing the constants*
 * <constant name> - the case-sensitive name of the constant as it appears after   *
 *                   the #define statement.                                        *
 * <operation> - a number of operations are supported:                             *
 *				1 - increment the constant by the <optional> value                 *
 *				2 - decrement the constant by the <optional> value                 *
 *              3 - change the constant to the <optional> value                    */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


int findconst( FILE *constfile, char constname[50] );


void main( int argc, char** argv )
{
	FILE *constfile;
	char constname[50];
	int operation, opvalue = 1, constvalue;

	if( argc > 4 )
	{
		constfile = fopen( argv[1], "r+" );
		strcpy( constname, argv[2] );
		operation = atoi(argv[3]);
		if( argc == 5 )
			opvalue = atoi(argv[4]);
	
		constvalue = findconst( constfile, constname );

		switch( operation )
		{
			case 1: constvalue += opvalue; break;
			case 2: constvalue -= opvalue; break;
			case 3: constvalue = opvalue; break;
		}

		fprintf(constfile, "#define %s\t%d", constname, constvalue);			
		fflush(constfile);
		fclose(constfile);
	}
	return;
}


int findconst( FILE *constfile, char constname[50] )
{
	int linenum, value;
	char buffer[300];
	char substring[300] = "#define ";

	strcat( substring, constname );

	fgets(buffer, 299, constfile );
	for( linenum = 0; !feof(constfile); linenum++ )
	{
		if( strncmp( substring, buffer, strlen(substring) ) == 0 )
		{
			fseek( constfile, (strlen(buffer)+1)*-1, SEEK_CUR );
			sscanf( buffer, "#define %s\t%d", substring, &value );
			return( value );
		}

		fgets(buffer, 299, constfile );
	}

	printf("Error! Constant not found!");
	fclose(constfile);
	exit(-1);
}

