/******************************************************************************
 *
 * Copyright (c) 1999-2000 Palm, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: Starter.c
 *
 * Release: Palm OS SDK 4.0 (63220)
 *
 *****************************************************************************/

#include <PalmOS.h>
#include "PalmAutoTestRsc.h"

#define cmdAutoGenMemos 99			//"hooks" to call GenMemo program to make memos
#define cmdAddMemo     100
#define ALARMTIME       60			//time allotted for synchronization phase of testing

/***********************************************************************
 *
 *	Entry Points
 *
 ***********************************************************************/


/***********************************************************************
 *
 *	Internal Structures
 *
 ***********************************************************************/
typedef struct 
	{
	UInt8 replaceme;
	} StarterPreferenceType;

typedef struct 
	{
	UInt8 replaceme;
	} StarterAppInfoType;

typedef struct							//structure with all the information needed for testing
{
	UInt8 operationtype;
	UInt16 nummemos;
	UInt16 memosize;
	UInt16 differences;
	UInt16 rounds;
	UInt16 bound;
	UInt8 installround;
} MemoGenInfo;

typedef StarterAppInfoType* StarterAppInfoPtr;


/***********************************************************************
 *
 *	Global variables
 *
 ***********************************************************************/
//static Boolean HideSecretRecords;


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define appFileCreator			'STRT'	// register your own at http://www.palmos.com/dev/creatorid/
#define appVersionNum			0x01
#define appPrefID				0x00
#define appPrefVersionNum		0x01

// Define the minimum OS version we support (2.0 for now).
#define ourMinVersion	sysMakeROMVersion(2,0,0,sysROMStageRelease,0)
#define kPalmOS10Version	sysMakeROMVersion(1,0,0,sysROMStageRelease,0)


/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/
void saveParameters( MemoGenInfo memogeninfo );


/***********************************************************************
 *
 * FUNCTION:    RomVersionCompatible
 *
 * DESCRIPTION: This routine checks that a ROM version is meet your
 *              minimum requirement.
 *
 * PARAMETERS:  requiredVersion - minimum rom version required
 *                                (see sysFtrNumROMVersion in SystemMgr.h 
 *                                for format)
 *              launchFlags     - flags that indicate if the application 
 *                                UI is initialized.
 *
 * RETURNED:    error code or zero if rom is compatible
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion)
		{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
			{
			FrmAlert (RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion <= kPalmOS10Version)
				{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
				}
			}
		
		return sysErrRomIncompatible;
		}

	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: This routine returns a pointer to an object in the current
 *              form.
 *
 * PARAMETERS:  formId - id of the form to display
 *
 * RETURNED:    void *
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void * GetObjectPtr(UInt16 objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}


/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr /*frmP*/)
{
}


/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormDoCommand(UInt16 command)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (command)
		{
		case MainOptionsAboutStarterApp:
			MenuEraseStatus(0);					// Clear the menu status from the display.
			frmP = FrmInitForm (AboutForm);
			FrmDoDialog (frmP);					// Display the About Box.
			FrmDeleteForm (frmP);
			handled = true;
			break;

		}
	
	return handled;
}


/* FUNCTION: getStoredValues                                                                        *
 * PURPOSE: This function retrieves the memo generation information from the application's database *
 * and returns it in the form of a static MemoGenInfo structure. If the memo generation information *
 * has not yet been created, then it is created here with the values that appear in the function.   *
 * THIS IS THE FUNCTION TO ENTER YOUR DESIRED INITIAL VALUES FOR A TEST RUN.                        *
 * DATE: 6/11/02                                                                                    */
MemoGenInfo getStoredValues()
{
	LocalID databaseid;
	DmOpenRef AutoTestDb;
	UInt16 dummy = 0;
	MemHandle parameters;
	MemoGenInfo *memogeninfoptr;
	MemoGenInfo memogeninfo;	
	
	databaseid = DmFindDatabase(0, "AutoTestDB");				//see if the database exists already
	AutoTestDb = DmOpenDatabase(0, databaseid, dmModeReadWrite );	
	parameters = DmGetRecord( AutoTestDb, 0 );
	memogeninfoptr = MemHandleLock( parameters );
	memogeninfo = *memogeninfoptr;		//copy the memo data to a location that will be available after we unlock and close the record and database
	MemHandleUnlock( parameters );
	DmReleaseRecord( AutoTestDb, 0, true );
	DmCloseDatabase( AutoTestDb );

	return(memogeninfo);
}


/* FUNCTION: callHotSync                                              *
 * PURPOSE: This function places a hotsync event in the event queue.  *
 * DATE: 6/11/02                                                      */
void callHotSync()
{
	EventType newEvent;

	newEvent.eType = keyDownEvent;
	newEvent.data.keyDown.chr = vchrHardCradle;
	newEvent.data.keyDown.modifiers = commandKeyMask;
	EvtAddEventToQueue(&newEvent);

	return;
}


/* FUNCTION: callGenMemo                                                                             *
 * PURPOSE: This function calls the GenMemo function to create memos given the MemoGenInfo structure *
 * (if action code = 0), or to add a specified memo to the memo list (if action code = 1).           *
 * DATE: 6/14/02                                                                                     */
void callGenMemo( MemoGenInfo memogeninfo, char *memotext, UInt8 actioncode )
{
	LocalID databaseid;
	UInt32 result;
	
	databaseid = DmFindDatabase( 0, "GenMemo" );
	if( databaseid )
	{
		if( actioncode == 0)								//use the MemoGenInfo structure to create random memos
			SysAppLaunch(0, databaseid, 0, cmdAutoGenMemos, &memogeninfo, &result);
		if( actioncode == 1)								//add the given memo text to the memo database
			SysAppLaunch(0, databaseid, 0, cmdAddMemo, memotext, &result);
	}
		
	return;
}


/* FUNCTION: updateParameters                                                                       *
 * PURPOSE: Given the memo generation information, this function will look at the type of operation *
 * being run, check to see if it will exceed its bounds, and will update the parameters for the next*
 * round appropriately. It will save the new parameters using the SaveParameters function           *
 * DATE: 6/12/02                                                                                    */
void updateParameters( MemoGenInfo memogeninfo )
{

	if( memogeninfo.installround )		//if this hotsync is used to install applications, don't update anything
		memogeninfo.installround = 0;	//but set the next hotsync to be a synchronization one
	else								//if this hotsync is used to synchronize
	{
		switch( memogeninfo.operationtype )
		{
			case 0: break;							//ERROR! This should not happen...
		
			case 1: memogeninfo.nummemos--;			//OPERATION 1: Decrement (by 1) number of memos generated 
					if( memogeninfo.nummemos <= memogeninfo.bound )
						memogeninfo.operationtype = 0; 
					break;
		
			case 2: memogeninfo.nummemos++;			//OPERATION 2: Increment (by 1) number of memos generated
					if( memogeninfo.nummemos >= memogeninfo.bound )
						memogeninfo.operationtype = 0; 
					break;
				
			case 3: memogeninfo.memosize -= 5; 		//OPERATION 3: Decrement memo sizes by 5
					if( memogeninfo.memosize <= memogeninfo.bound )
						memogeninfo.operationtype = 0; 
					break;
				
			case 4: memogeninfo.memosize += 5;		//OPERATION 4: Increment memo sizes by 5
					if( memogeninfo.memosize >= memogeninfo.bound )
						memogeninfo.operationtype = 0; 
					break;
				
			case 8:									//OPERATION 8: Keep parameters the same... the constants are changing on the PC side
			case 5:                                 //OPERATION 5: Keep the parameters constant, just repeat for a certain # of rounds
			case 6: memogeninfo.rounds++;			//OPERATION 6: "Smoke Test" - same as operation 5, just with a different operation number
					if( memogeninfo.rounds >= memogeninfo.bound )
						memogeninfo.operationtype = 0;
					break;
					
			case 7: memogeninfo.nummemos += 25;		//OPERATION 7: Add 25 to the number of memos generated
					if( memogeninfo.nummemos >= memogeninfo.bound )
						memogeninfo.operationtype = 0;
					break;
					
			case 10: memogeninfo.differences += 4;	//OPERATION 10: Increase the differences between the data sets by 4
					if( memogeninfo.differences >= memogeninfo.bound )
						memogeninfo.operationtype = 0;
					break;
					
			
		}
	}
	saveParameters( memogeninfo );
}
	
	
/* FUNCTION: saveParameters                                                            *
 * PURPOSE: Given a pointer to the memo generation parameters, this function will save *
 * them to the "AutoTestDB" database.                                                  *
 * DATE: 7/17/02                                                                       */
void saveParameters( MemoGenInfo memogeninfo )
{
	LocalID databaseid;
	DmOpenRef AutoTestDb;
	UInt16 dummy = 0;
	MemHandle parameters;
	MemoGenInfo *memogeninfoptr;


	databaseid = DmFindDatabase(0, "AutoTestDB");			
	AutoTestDb = DmOpenDatabase(0, databaseid, dmModeReadWrite );
	parameters = DmGetRecord( AutoTestDb, 0 );
	memogeninfoptr = MemHandleLock( parameters );
	DmWrite( memogeninfoptr, 0, &memogeninfo, sizeof( memogeninfo ) );
	MemHandleUnlock( parameters );
	DmReleaseRecord( AutoTestDb, 0, true );
	DmCloseDatabase( AutoTestDb );
	return;
}
			

/* FUNCTION: setMyAlarm                                                                      *
 * PURPOSE: This function sets an "alarm" to this application that will execute in ALARMTIME * 
 * seconds from now... at that time, the application will wake up with the                   *
 * sysAppLaunchCmdAlarmTriggered launch flag.                                                *
 * DATE: 6/12/02                                                                             */
void setMyAlarm( UInt16 alarmtime )
{
	LocalID databaseid;
	
	databaseid = DmFindDatabase(0, "PalmAutoTest");
	AlmSetAlarm( 0, databaseid, 0, TimGetSeconds() + alarmtime, false );
}


/* FUNCTION: checkStoredValues                                                                *
 * PURPOSE: This function will check to see if the AutoTestDB database exists. This is a good *
 * indicator of whether or not we are actually in the middle of an automated test (if the     *
 * database exists) or if we have somehow arrived here erroneously (if the PalmAutoTest was   *
 * included in a hotsync).                                                                    *
 * DATE: 7/3/02                                                                               */
UInt8 checkStoredValues()
{
	LocalID databaseid;
	
	databaseid = DmFindDatabase(0, "AutoTestDB");				//see if the database exists already
	if( databaseid )
		return 1;
	else
		return 0;
}


/* FUNCTION: palmAutoTest                                                                  *
 * PURPOSE: This function defines what the palm does at each "round" in an automated test. *
 * The steps are: 1) Restoring the stored values for memo generation 2) Using the GenMemo  *
 * application to generate a certain number of memos. 3) Placeing a hotsync event on the   *
 * event queue. This function will also list what the daily smoke test does at each round  *
 * based on the switch statement.                                                          *
 * DATE: 6/14/02                                                                           */
void palmAutoTest()
{
	MemoGenInfo memogeninfo;

	memogeninfo = getStoredValues();
	
	if( memogeninfo.installround == 1 )		//every other round we must hotsync to install modified apps
	{
		setMyAlarm( 45 );
		updateParameters( memogeninfo );
		callHotSync();
		return;
	}
	
	if( memogeninfo.operationtype != 0 )			//if we are not done with the automated run
	{
		setMyAlarm( ALARMTIME );
		if( memogeninfo.operationtype != 6 )		//if we are not in a smoke test
		{
			callGenMemo( memogeninfo, NULL, 0 );		//just add random memos according to the memogeninfo structure
		}
		else										//if we are in a smoke test
		{
			switch( memogeninfo.rounds )				//add specific memos depending on the particular smoke test round
			{
				case 0: callGenMemo( memogeninfo, "This is a test memo.", 1); break;
				case 2: callGenMemo( memogeninfo, "Boston University is located in Boston Mass.", 1); break;
			}
		}
		
		updateParameters( memogeninfo );
		callHotSync();
	}
}

/* FUNCTION: initializeAutoTest                                                    *
 * PURPOSE: This function will initialize the parameters for automatic testing.    *
 * DATE: 7/15/02                                                                   */
void initializeAutoTest()
{
	LocalID databaseid;
	DmOpenRef AutoTestDb;
	UInt16 dummy = 0;
	MemHandle parameters;
	MemoGenInfo *memogeninfoptr;
	MemoGenInfo memogeninfo;	
	
	databaseid = DmFindDatabase( 0, "SyncDocDB" );				//delete all memos from the Palm
	if (databaseid)
		DmDeleteDatabase( 0, databaseid );

	databaseid = DmFindDatabase( 0, "SyncIDocDB" );
	if (databaseid)
		DmDeleteDatabase( 0, databaseid );
						
	databaseid = DmFindDatabase( 0, "SyncHashDB" );
	if (databaseid)
		DmDeleteDatabase( 0, databaseid );
						
	databaseid = DmFindDatabase( 0, "SyncPolyDB" );
	if (databaseid)
		DmDeleteDatabase( 0, databaseid );

	
	databaseid = DmFindDatabase(0, "AutoTestDB");				//see if the database exists already
	if (databaseid)												//if it does, then it contains old values, and we should delete them
		DmDeleteDatabase( 0, databaseid );

	DmCreateDatabase(0, "AutoTestDB", 'STRT', 0, false );		//if it doesn't, then create it, and get a pointer to the initialized values
	databaseid = DmFindDatabase(0, "AutoTestDB");
	AutoTestDb = DmOpenDatabase(0, databaseid, dmModeReadWrite );
	parameters = DmNewRecord( AutoTestDb, &dummy , sizeof(memogeninfo)*2  );		
	memogeninfoptr = MemHandleLock( parameters );
	memogeninfo.operationtype = 7;								//THESE ARE THE INITIAL VALUES FOR THE AUTO TEST.. PLACE DESIRED VALUES HERE
	memogeninfo.nummemos = 100;
	memogeninfo.memosize = 120;
	memogeninfo.differences = 0;
	memogeninfo.rounds = 0;												
	memogeninfo.bound = 350;
	memogeninfo.installround = 1;						//the first hotsync will install the Palm apps
	DmWrite( memogeninfoptr, 0, &memogeninfo, sizeof( memogeninfo ) );	//write the initial values to a database	
	MemHandleUnlock( parameters );
	DmReleaseRecord( AutoTestDb, 0, true );
	DmCloseDatabase( AutoTestDb );
}


/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr eventP)
{
	Boolean handled = false;
	FormPtr frmP;

	switch (eventP->eType) 
		{
		case menuEvent:
			return MainFormDoCommand(eventP->data.menu.itemID);

		case frmOpenEvent:
			frmP = FrmGetActiveForm();
			MainFormInit( frmP);
			FrmDrawForm ( frmP);
			handled = true;
			break;
			
		case frmUpdateEvent:
			// To do any custom drawing here, first call FrmDrawForm(), then do your
			// drawing, and then set handled to true.
			break;
			

		case ctlSelectEvent:
			if( eventP ->data.ctlSelect.controlID == Main_Start_Button )		//EVENT: "Start Automated Test" button pressed
			{
				initializeAutoTest();
				setMyAlarm(20);						//set the alarm to start the autotest in 3 seconds
				handled = true;
			}

		default:
			break;
		
		}
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Boolean AppHandleEvent(EventPtr eventP)
{
	UInt16 formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent)
		{
		// Load the form resource.
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId)
			{
			case MainForm:
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;

			default:
//				ErrFatalDisplay("Invalid Form Load Event");
				break;

			}
		return true;
		}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static Err AppStart(void)
{
	StarterPreferenceType prefs;
	UInt16 prefsSize;

	// Read the saved preferences / saved-state information.
	prefsSize = sizeof(StarterPreferenceType);
	if (PrefGetAppPreferences(appFileCreator, appPrefID, &prefs, &prefsSize, true) != 
		noPreferenceFound)
		{
		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void AppStop(void)
{
	StarterPreferenceType prefs;

	// Write the saved preferences / saved-state information.  This data 
	// will saved during a HotSync backup.
	PrefSetAppPreferences (appFileCreator, appPrefID, appPrefVersionNum, 
		&prefs, sizeof (prefs), true);
		
	// Close all the open forms.
	FrmCloseAllForms ();
}


/***********************************************************************
 *
 * FUNCTION:    StarterPalmMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static UInt32 StarterPalmMain(UInt16 cmd, MemPtr /*cmdPBP*/, UInt16 launchFlags)
{
	Err error;

	error = RomVersionCompatible (ourMinVersion, launchFlags);
	if (error) return (error);

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;


		case sysAppLaunchCmdAlarmTriggered:
			palmAutoTest();
			break;
			
		default:
			break;

		}
	
	return errNone;
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
UInt32 PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
	return StarterPalmMain(cmd, cmdPBP, launchFlags);
}
